function w=heat(f,g1,g2,L,T,N,M,a)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Equazione del calore in una dimensione: 
%    u_t = a * u_xx
%    IC:  u(x,0)=f(x)       0 <= x <= L
%    BC:  u(0,t)=g1(t)    u(L,t)=g2(t)
%    diffusione del calore 
%    in un filo metallico di lunghezza L
%	  termicamente isolato con distribuzione 
%    iniziale della temperatura nota f(t). 
%    La sol. u(x,t) rappresenta la distribuzione 
%    della temperatura nei T istanti successivi a 
%    quello iniziale.
%    METODO ESPLICITO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
%

% h = passo spaziale in  x
h=L/N;
% k = passo temporale in t
k=T/M;
% M numero di passi in x 
% N numero di passi in t
r=a* k/(h^2);
rr=1-2*r;
x=h:h:(N-1)*h;
t=k:k:M*k;

%valuta le condizioni iniziali  IC:
    y0=f(0);  y=f(x);   yL=f(L);

%valuta le condizioni al contorno BC:
    g1k=g1(t);  g2k=g2(t);
    
% trova la soluzione al primo passo temporale:
u(1,1)=r*y0+rr*y(1)+r*y(2);
u(2:N-2,1)=r*y(1:N-3)'+rr*y(2:N-2)'+r*y(3:N-1)';
u(N-1,1)=r*y(N-2)+rr*y(N-1)+r*yL;

% trova la soluzione nei rimanenti passi temporali:
for j=2:M
    u(1,j)=r*g1k(j-1)+rr*u(1,j-1)+r*u(2,j-1);
    u(2:N-2,j)=r*u(1:N-3,j-1)+rr*u(2:N-2,j-1)+r*u(3:N-1,j-1);
    u(N-1,j)=r*u(N-2,j-1)+rr*u(N-1,j-1)+r*g2k(j-1);
end

%w=[g1k u' g2k
%    y0  y yL];
w=[y0  y yL
   g1k u' g2k];
   