function w=heat_esp(f,g1,g2,L,T,N,M,a)
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% Equazione del calore in una dimensione: 
%    u_t = a * u_xx
%    IC:  u(x,0)=f(x)       0 <= x <= L
%    BC:  u(0,t)=g1(t)    u(L,t)=g2(t)
%    diffusione del calore 
%    in un filo metallico di lunghezza L
%	  termicamente isolato con distribuzione 
%    iniziale della temperatura nota f(t). 
%    La sol. u(x,t) rappresenta la distribuzione 
%    della temperatura nei T istanti successivi a 
%    quello iniziale.
%    METODO ESPLICITO
% ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% h = passo spaziale in  x
h=L/(N+1);
% N+1=numero intervalli N numero nodi interni
% k = passo temporale in t
k=T/M;
r=a* k/(h^2);
rr=1-2*r;

x=h:h:N*h;
t=0:k:M*k;
% %valuta le condizioni iniziali  IC:
u(:,1)=f(x);
% %valuta le condizioni al contorno BC:
g1k=g1(t);  g2k=g2(t);
% matrice dei nodi interni (NxN)
A=+diag(r*ones(N-1,1),-1)+diag(rr*ones(N,1))+diag(r*ones(N-1,1),+1);
for j=1:M
    u(:,j+1)=A*u(:,j);
    u(1,j+1)=u(1,j+1)+r*g1k(j);
    u(N,j+1)=u(N,j+1)+r*g2k(j);
end
w= [g1k u' g2k];
    